__version__ = '0.1.6'
__title__ = "Waf Web Finder"
__description__ = "Automated web server (behind WAF) finder."
__url__ = "https://github.com/helviojunior/webfinder"
__build__ = 0x63c974a8
__author__ = "Helvio Junior (M4v3r1ck)"
__author_email__ = "helvio_junior@hotmail.com"
__license__ = "GPL-3.0"
__copyright__ = "Copyright Helvio Junior"



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

from ..util.tools import Tools

import os, subprocess, socket, re, queue, threading, sys, time, json, hashlib

from ..config import Configuration
from ..util.logger import Logger
from ..util.getter import Getter

class PathGetter:

    words = []
    q = queue.Queue()
    added = []
    last_start = []
    ingore_until = ''
    current_gettter = None
    current_uri = ''
    running=True
    duplicated=0
    last_item=""
    paused=True
    skip_current=False

    def __init__(self):
        pass

    def load_iplist(self):

        if Configuration.threads_data is not None:
            try:
                for i in Configuration.threads_data:
                    self.last_start.append(Configuration.threads_data[i])
            except:
                pass

        with open(Configuration.ip_list, 'r', encoding="ascii", errors="surrogateescape") as f:
            line = f.readline()
            while line:
                if line.endswith('\n'):
                    line = line[:-1]
                if line.endswith('\r'):
                    line = line[:-1]

                line = ''.join(filter(Tools.permited_char, line)).strip()

                if self.ingore_until == '' and line in self.last_start:
                    self.ingore_until = line     

                if line not in Configuration.ipaddresses:
                    Configuration.ipaddresses.append(line.strip('\r\n\t').strip())
                    self.last_item = line
                #else:
                #    self.duplicated+=1

                try:
                    line = f.readline()
                except:
                    pass

    def len(self):
        return len(Configuration.ipaddresses)


    def run(self):

        t = threading.Thread(target=self.worker)
        t.daemon = True
        t.start()

        t_status = threading.Thread(target=self.status_worker)
        t_status.daemon = True
        t_status.start()

        count = 0
        with self.q.mutex:
            self.q.queue.clear()

        count += 1
        self.q.put(Configuration.target)

        #if len(list(self.q.queue)) > 0:
        if (count > 0):
            self.paused=False

            self.q.join()  # block until all tasks are done
            sys.stdout.write("\033[K")  # Clear to the end of line

    def worker(self):
        try:
            while self.running:

                while self.paused:
                    time.sleep(0.3)

                item = self.q.get()

                self.current_uri = item
                self.current_gettter = Getter(Configuration.ipaddresses, False)
                self.current_gettter.ingore_until = self.ingore_until
                self.current_gettter.run(item)

                if Configuration.verbose > 3:
                    Logger.pl('\n{*} {W}Finishing %s{W}' % item)

                self.ingore_until = ''
                self.q.task_done()
        except KeyboardInterrupt as e:
            raise e

    def testing_base(self):
        return self.current_uri == Configuration.target

    def pause(self):
        self.paused=True
        self.running=False
        self.save_status()
        self.current_gettter.pause()
        
        
    def skip(self):
        self.ingore_until = ''
        self.save_status(True)
        self.running=False
        self.paused=False
        self.current_gettter.stop()
        with self.q.mutex:
            self.q.queue.clear()


    def save_status(self, skip_current=False):
        paths_found = self.current_gettter.path_found

        for u in paths_found:
            if u.endswith('/'):
                u = u[:-1]
            if u not in self.added:
                self.added.append(u)
                self.q.put(u)


        dt = { 
        "command" : Configuration.cmd_line,
        "current_path" : self.current_uri,
        "skip_current" : skip_current,
        "paths": self.added,
        "deep_links": Getter.deep_links,
        "threads": self.current_gettter.last 
         }

        with open("webfinder.restore", "w") as text_file:
            text_file.write(json.dumps(dt))

    def status_worker(self):
        try:
            while self.running:
                try:
                    if self.current_gettter is None:
                        time.sleep(1)
                        continue

                    if self.paused:
                        time.sleep(1)
                        continue

                    self.save_status()

                except:
                    raise
                time.sleep(10)
        except KeyboardInterrupt:
            pass
            




#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import sys
import colorama
from colorama import Fore, Back, Style
colorama.init()

class Color(object):
    ''' Helper object for easily printing colored text to the terminal. '''

    # Basic console colors
    colors = {
        'W' : '\033[0m',  # white (normal)
        'R' : '\033[31m', # red
        'G' : '\033[32m', # green
        'O' : '\033[33m', # orange
        'B' : '\033[34m', # blue
        'P' : '\033[35m', # purple
        'C' : '\033[36m', # cyan
        'GR': '\033[37m', # gray
        'D' : '\033[2m'   # dims current color. {W} resets.
    }

    # Helper string replacements
    replacements = {
        '{+}': ' {W}{D}[{W}{G}+{W}{D}]{W}',
        '{!}': ' {O}[{R}!{O}]{W}',
        '{?}': ' {W}[{C}?{W}]',
        '{*}': ' {W}[{B}*{W}]'
    }

    last_sameline_length = 0

    @staticmethod
    def p(text):
        '''
        Prints text using colored format on same line.
        Example:
            Color.p("{R}This text is red. {W} This text is white")
        '''
        sys.stdout.write(Color.s(text))
        sys.stdout.flush()
        if '\r' in text:
            text = text[text.rfind('\r')+1:]
            Color.last_sameline_length = len(text)
        else:
            Color.last_sameline_length += len(text)

    @staticmethod
    def pl(text):
        '''Prints text using colored format with trailing new line.'''
        Color.p('%s\n' % text)
        Color.last_sameline_length = 0

    @staticmethod
    def pe(text):
        '''Prints text using colored format with leading and trailing new line to STDERR.'''
        sys.stderr.write(Color.s('%s\n' % text))
        Color.last_sameline_length = 0

    @staticmethod
    def s(text):
        ''' Returns colored string '''
        output = text
        for (key,value) in Color.replacements.items():
            output = output.replace(key, value)
        for (key,value) in Color.colors.items():
            output = output.replace("{%s}" % key, value)
        return output

    @staticmethod
    def sc(text):
        ''' Returns non colored string '''
        output = text
        for (key,value) in Color.replacements.items():
            output = output.replace(key, value)
        for (key,value) in Color.colors.items():
            output = output.replace("{%s}" % key, '')
        return output

    @staticmethod
    def clear_line():
        spaces = ' ' * Color.last_sameline_length
        sys.stdout.write('\r%s\r' % spaces)
        sys.stdout.flush()
        Color.last_sameline_length = 0

    @staticmethod
    def clear_entire_line():
        import os
        (rows, columns) = os.popen('stty size', 'r').read().split()
        Color.p("\r" + (" " * int(columns)) + "\r")

    @staticmethod
    def pattack(attack_type, target, attack_name, progress):
        '''
        Prints a one-liner for an attack.
        Includes attack type (WEP/WPA), target ESSID & power, attack type, and progress.
        ESSID (Pwr) Attack_Type: Progress
        e.g.: Router2G (23db) WEP replay attack: 102 IVs
        '''
        essid = "{C}%s{W}" % target.essid if target.essid_known else "{O}unknown{W}"
        Color.p("\r{+} {G}%s{W} ({C}%sdb{W}) {G}%s {C}%s{W}: %s " % (
            essid, target.power, attack_type, attack_name, progress))

if __name__ == '__main__':
    Color.pl("{R}Testing{G}One{C}Two{P}Three{W}Done")
    print(Color.s("{C}Testing{P}String{W}"))
    Color.pl("{+} Good line")
    Color.pl("{!} Danger")



#!/usr/bin/python3
# -*- coding: UTF-8 -*-

import string, random, sys, re
from urllib.parse import urlparse


class Tools:

    def __init__(self):
        pass

    @staticmethod
    def random_generator(size=6, chars=string.ascii_uppercase + string.digits):
        return ''.join(random.choice(chars) for x in range(size))


    @staticmethod
    def clear_line():
        sys.stdout.write("\033[K")  # Clear to the end of line

    @staticmethod     
    def permited_char(s):
        if s.isalpha():
            return True
        elif bool(re.match("^[A-Za-z0-9:]*$", s)):
            return True
        elif s == ".":
            return True
        else:
            return False

    @staticmethod
    def get_host(url):
        rUri = urlparse(url)
        host = rUri.netloc.strip(': ')
        if ':' in host:
            host = host.split(':')[0]
        return host

    @staticmethod
    def get_port(url):
        rUri = urlparse(url)
        port = ''
        if rUri.scheme.lower() == 'https':
            port = 443
        else:
            port = 80
        if ':' in rUri.netloc:
            port = rUri.netloc.split(':')[1]
        return port

    @staticmethod
    def get_entropy(data: [str, bytes]) -> float:
        # https://en.wiktionary.org/wiki/Shannon_entropy
        from math import log2

        if data is None:
            return 0.0

        if isinstance(data, str):
            data = data.encode('utf-8', 'ignore')

        if len(data) == 0:
            return 0.0

        counters = {byte: 0 for byte in range(2 ** 8)}  # start all counters with zeros

        for byte in data:
            counters[byte] += 1

        probabilities = [counter / len(data) for counter in counters.values()]  # calculate probabilities for each byte

        entropy = -sum(
            probability * log2(probability) for probability in probabilities if probability > 0)  # final sum

        return round(entropy, 2)